'''
Advanced Power Abstract Wrappers
2008-2010(c) Kirill Plyashkevich <ru.spirit@gmail.com>
'''

import dbus
import Util
from Util import System, Shell

class AbstractWrapper():

  def __init__(self, bus, check = False):
    self.bus = bus
    self.dbus_signals = {}
    self.export_signals = {}
    self.export_interfaces = {}
    self.interfaces = {}
    self.interfaces_map = {}
    self.dep_packages = []
    self.friendly_name = ''
    self.available = False
    self.pre_init()
    self.check_available()
    if not check:
      self.end_init()

  def end_init(self):
    if self.available:
      self.dbus_signals = {'NameOwnerChanged': {'source': (self.dbus_name_owner_handler, 'NameOwnerChanged', 'org.freedesktop.DBus', '/org/freedesktop/DBus')}}
      self.ifaces = {}
      for iface in self.interfaces_map: self.ifaces[self.interfaces_map[iface][0]] = iface
      self.init_interfaces()
      self.init_dbus_signals()
      self.post_init()
      self.get_status()

  def pre_init(self):
    raise NotImplementedError("AbstractWrapper.pre_init() must be implemented")

  def post_init(self):
    raise NotImplementedError("AbstractWrapper.post_init() must be implemented")

  def dbus_name_owner_handler(self, name, old, new):
    name = str(name)
    if name in self.ifaces:
      if str(new):
        self.init_interface(self.ifaces[name])
      else:
        self.destroy_interface(self.ifaces[name])

  def get_status(self):
    raise NotImplementedError("AbstractWrapper.get_status() must be implemented")

  def _cleanup(self):
    self.destroy_dbus_signals()
    self.destroy_interfaces()

  def get_dep_packages(self):
    return self.dep_packages

  def get_name(self):
    return self.friendly_name

  def get_signals(self):
    return self.export_signals

  def is_available(self):
    return self.available

  def check_available(self):
    self.available = True
    for dp in self.dep_packages:
      if not System.is_package_version_valid(dp, self.dep_packages[dp]):
        self.available = False
        break
    return self.available

  def custom_available(self):
    return True

  def init_interfaces(self):
    for x in self.interfaces_map: self.init_interface(x)

  def init_interface(self, x):
    try:
      self.interfaces[x] = dbus.Interface(self.bus.get_object(self.interfaces_map[x][0], self.interfaces_map[x][1]), self.interfaces_map[x][2])
    except:
      self.interfaces[x] = None

  def destroy_interfaces(self):
    for key in self.interfaces: self.destroy_interface(key)

  def destroy_interface(self, x):
    self.interfaces[x] = None

  def init_dbus_signals(self):
    for key in self.dbus_signals: self.dbus_signals[key]['receiver_match'] = self.bus.add_signal_receiver(self.dbus_signals[key]['source'][0], signal_name=self.dbus_signals[key]['source'][1], dbus_interface=self.dbus_signals[key]['source'][2], path=self.dbus_signals[key]['source'][3])

  def destroy_dbus_signals(self):
    for key in self.dbus_signals:
      receiver_match = self.dbus_signals[key]['receiver_match']
      self.bus._clean_up_signal_match(receiver_match)
      self.bus.remove_signal_receiver(receiver_match)
      receiver_match.remove()
